package cn.jx.wechat.app.wechatrobot.config;


import cn.jx.wechat.app.wechatrobot.util.SpringBeanHelper;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;

import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnWebApplication;
import org.springframework.boot.json.GsonJsonParser;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.DependsOn;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.scheduling.support.CronTrigger;
import org.springframework.web.client.RestTemplate;

import java.lang.reflect.Type;
import java.time.Duration;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;



@Configuration
public class BusinessAccessTokenConfig {

    private String  accessToken;
    private Boolean  tokenEffect=false;
    // 正常调用gettoken接口不得超过15000次/小时，1000次/分钟
    private final static int maxCallTokenTimesPerMinute=10000;
    private final AtomicInteger   currentCallTokenTimes=new AtomicInteger(0);
    private final static  String Access_Token_Url="https://qyapi.weixin.qq.com/cgi-bin/gettoken?corpid=%s&corpsecret=%s";

    private static Logger log= LoggerFactory.getLogger(BusinessAccessTokenConfig.class);

    public BusinessAccessTokenConfig(@Qualifier("businessScheduler") ThreadPoolTaskScheduler  businessScheduler){
        businessScheduler.schedule(() -> {
            log.info("每1分钟校验企业微信accessToken[开始]>>>>");
            init();
            log.info("每1分钟校验企业微信accessToken[结束]<<<<<");
        }, new CronTrigger("0 0/1 * * * ?"));

    }

    protected void init(){
        try {
            BusinessVerifyProperties businessVerifyProp = SpringBeanHelper.getBean(BusinessVerifyProperties.class);
            String accessTokenUrl = String.format(Access_Token_Url, businessVerifyProp.getReceiveid(), businessVerifyProp.getCorpSecret());
            RestTemplate rest = SpringBeanHelper.getBean("defaultAppRest", RestTemplate.class);
            if(accessToken == null || accessToken.isEmpty()){
                if(currentCallTokenTimes.get()<maxCallTokenTimesPerMinute){
                    currentCallTokenTimes.incrementAndGet();
                    ResponseEntity<String> responseEntity = rest.exchange(accessTokenUrl, HttpMethod.GET, HttpEntity.EMPTY, String.class);
                    if (responseEntity.getStatusCode().is2xxSuccessful()) {
                        Gson gson = new GsonBuilder().setPrettyPrinting().create();
                        Type type = new TypeToken<Map<String, String>>() {
                        }.getType();
                        Map<String,String> fromJson = gson.fromJson(responseEntity.getBody(), type);
                        if (null!=fromJson&&fromJson.containsKey("errcode")&&fromJson.get("errcode").equals("0")) {
                            this.accessToken = fromJson.get("access_token");
                            this.tokenEffect=true;
                            log.info("企业微信accessToken:{} 在有效期内", accessToken);
                        } else if(null!=fromJson&&fromJson.containsKey("errcode")&&fromJson.get("errcode").equals("42001")) {
                            this.accessToken=null;
                            this.tokenEffect=false;
                            this.init();
                        }
                    } else{
                        this.accessToken=null;
                        this.tokenEffect=false;
                        this.init();
                    }
                }
            }else{
                if(currentCallTokenTimes.get()<maxCallTokenTimesPerMinute){
                    ResponseEntity<String> responseEntity = rest.exchange(accessTokenUrl, HttpMethod.GET, HttpEntity.EMPTY, String.class);
                    if (responseEntity.getStatusCode().is2xxSuccessful()) {
                        Gson gson = new GsonBuilder().setPrettyPrinting().create();
                        Type type = new TypeToken<Map<String, String>>() {
                        }.getType();
                        Map<String,String> fromJson = gson.fromJson(responseEntity.getBody(), type);
                        if (null!=fromJson&&fromJson.containsKey("errcode")&&fromJson.get("errcode").equals("0")) {
                            if(null!=this.accessToken&& !this.accessToken.isEmpty() &&fromJson.get("access_token").equals(this.accessToken)){
                                if(fromJson.containsKey("expires_in")
                                        &&null!=fromJson.get("expires_in")
                                        &&!"".equals(fromJson.get("expires_in"))
                                        &&Integer.parseInt(fromJson.get("expires_in"))<7200
                                ){
                                    log.info("企业微信accessToken:{} 在有效期内", this.accessToken);
                                }else{
                                    this.accessToken=null;
                                    this.tokenEffect=false;
                                    this.init();
                                }
                            }else{
                                this.accessToken=null;
                                this.tokenEffect=false;
                                this.init();
                            }
                        } else if(null!=fromJson&&fromJson.containsKey("errcode")&&fromJson.get("errcode").equals("42001")) {
                            this.accessToken=null;
                            this.tokenEffect=false;
                            this.init();
                        }
                    } else{
                        this.accessToken=null;
                        this.tokenEffect=false;
                        this.init();
                    }
                }
            }

        }catch (Exception e){
            //ignore
            this.accessToken=null;
            this.tokenEffect=false;
        }
    }

    public String currentAccessToken(){
        if(this.accessToken==null||this.accessToken.isEmpty()){
            throw new RuntimeException("暂无权限,稍后重试");
        }else {
            if( this.tokenEffect){
                return this.accessToken;
            }else {
                throw new RuntimeException("暂无权限,稍后重试");
            }
        }
    }

}
