package cn.jx.wechat.app.wechatrobot.util;



//import javax.crypto.Cipher;
//import javax.crypto.spec.IvParameterSpec;
//import javax.crypto.spec.SecretKeySpec;
import org.bouncycastle.util.encoders.Base64;

import javax.crypto.Cipher;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.util.Arrays;

public class WXBizMsgCrypt {

    static Charset CHARSET = StandardCharsets.UTF_8;
    private String  token;
    private String  receiveid;
    private byte[]  aesKey;
    /**
     * 构造函数
     * @param token 企业微信后台，开发者设置的token
     * @param encodingAesKey 企业微信后台，开发者设置的EncodingAESKey
     * @param receiveid, 不同场景含义不同，详见文档
     */
    public WXBizMsgCrypt(String token, String encodingAesKey, String receiveid) {
        if (encodingAesKey.length() != 43) {
            throw new RuntimeException("不合法的encodingAesKey参数");
        }

        this.token = token;
        this.receiveid = receiveid;
        aesKey = Base64.decode(encodingAesKey + "=");
    }


    public   String VerifyURL(String msgSignature, String timeStamp, String nonce, String echoStr) {
        String signature = SHA1.getSHA1(token, timeStamp, nonce, echoStr);
        if (!signature.equals(msgSignature)) {
            throw new RuntimeException("不合法的消息签名");
        }
        return decrypt(echoStr);
    }




    /**
     * 对密文进行解密.
     *
     * @param text 需要解密的密文
     * @return 解密得到的明文
     */
    String decrypt(String text)  {
        byte[] original;
        try {
            // 设置解密模式为AES的CBC模式
            Cipher cipher = Cipher.getInstance("AES/CBC/NoPadding");
            SecretKeySpec key_spec = new SecretKeySpec(aesKey, "AES");
            IvParameterSpec iv = new IvParameterSpec(Arrays.copyOfRange(aesKey, 0, 16));
            cipher.init(Cipher.DECRYPT_MODE, key_spec, iv);

            // 使用BASE64对密文进行解码
            byte[] encrypted = Base64.decode(text);

            // 解密
            original = cipher.doFinal(encrypted);
        } catch (Exception e) {
            throw new RuntimeException("消息签名验证失败-[提取身份信息异常]");
        }

        String xmlContent, from_receiveid;
        try {

            // 去除补位字符
            byte[] bytes = PKCS7Encoder.decode(original);

            // 分离16位随机字符串,网络字节序和receiveid
            byte[] networkOrder = Arrays.copyOfRange(bytes, 16, 20);

            int xmlLength = recoverNetworkBytesOrder(networkOrder);

            xmlContent = new String(Arrays.copyOfRange(bytes, 20, 20 + xmlLength), CHARSET);
            from_receiveid = new String(Arrays.copyOfRange(bytes, 20 + xmlLength, bytes.length),
                    CHARSET);
        } catch (Exception e) {
            throw new RuntimeException("消息签名验证失败-[身份信息无法正常获取]");
        }

        // receiveid不相同的情况
        if (!from_receiveid.equals(receiveid)) {
            throw new RuntimeException("消息签名验证失败-[身份来源不一致]");
        }
        return xmlContent;

    }


    // 还原4个字节的网络字节序
    int recoverNetworkBytesOrder(byte[] orderBytes) {
        int sourceNumber = 0;
        for (int i = 0; i < 4; i++) {
            sourceNumber <<= 8;
            sourceNumber |= orderBytes[i] & 0xff;
        }
        return sourceNumber;
    }










    /**
     * SHA1 class
     * 计算消息签名接口.
     */
    static class SHA1 {

        /**
         * 用SHA1算法生成安全签名
         * @param token 票据
         * @param timestamp 时间戳
         * @param nonce 随机字符串
         * @param encrypt 密文
         * @return 安全签名
         */
        public static String getSHA1(String token, String timestamp, String nonce, String encrypt)
        {
            try {
                String[] array = new String[] { token, timestamp, nonce, encrypt };
                StringBuilder sb = new StringBuilder();
                // 字符串排序
                Arrays.sort(array);
                for (int i = 0; i < 4; i++) {
                    sb.append(array[i]);
                }
                String str = sb.toString();
                // SHA1签名生成
                MessageDigest md = MessageDigest.getInstance("SHA-1");
                md.update(str.getBytes());
                byte[] digest = md.digest();

                StringBuilder hexstr = new StringBuilder();
                String shaHex = "";
                for (int i = 0; i < digest.length; i++) {
                    shaHex = Integer.toHexString(digest[i] & 0xFF);
                    if (shaHex.length() < 2) {
                        hexstr.append(0);
                    }
                    hexstr.append(shaHex);
                }
                return hexstr.toString();
            } catch (Exception e) {
                throw new RuntimeException("无法计算的消息签名");
            }
        }
    }




    /**
     * 提供基于PKCS7算法的加解密接口.
     */
   static class PKCS7Encoder {
        static Charset CHARSET = StandardCharsets.UTF_8;
        static int BLOCK_SIZE = 32;

        /**
         * 获得对明文进行补位填充的字节.
         *
         * @param count 需要进行填充补位操作的明文字节个数
         * @return 补齐用的字节数组
         */
        static byte[] encode(int count) {
            // 计算需要填充的位数
            int amountToPad = BLOCK_SIZE - (count % BLOCK_SIZE);
            if (amountToPad == 0) {
                amountToPad = BLOCK_SIZE;
            }
            // 获得补位所用的字符
            char padChr = chr(amountToPad);
            String tmp = new String();
            for (int index = 0; index < amountToPad; index++) {
                tmp += padChr;
            }
            return tmp.getBytes(CHARSET);
        }

        /**
         * 删除解密后明文的补位字符
         *
         * @param decrypted 解密后的明文
         * @return 删除补位字符后的明文
         */
        static byte[] decode(byte[] decrypted) {
            int pad = (int) decrypted[decrypted.length - 1];
            if (pad < 1 || pad > 32) {
                pad = 0;
            }
            return Arrays.copyOfRange(decrypted, 0, decrypted.length - pad);
        }

        /**
         * 将数字转化成ASCII码对应的字符，用于对明文进行补码
         *
         * @param a 需要转化的数字
         * @return 转化得到的字符
         */
        static char chr(int a) {
            byte target = (byte) (a & 0xFF);
            return (char) target;
        }

    }

}
